@@ -19,13 +19,16 @@ if test -f config.h; then :; else
 */
 
 #define PACKAGE "sed"
-#define VERSION "4.0.8-boot"
+#define VERSION "4.0.9-boot"
 #define SED_FEATURE_VERSION "4.0"
 #define BOOTSTRAP 1
 
 /* Define if your compiler/headers don't support const. */
 #undef const
 
+/* Undefine if headers have conflicting definition.  */
+#define mbstate_t int
+
 /* Toggle if you encounter errors in lib/mkstemp.c.  */
 #define HAVE_UNISTD_H
 #define HAVE_FCNTL_H
@@ -46,7 +49,7 @@ END_OF_CONFIG_H
 fi
 
 # tell the user what we're doing from here on...
-set -x
+set -x -e
 
 # the ``|| exit 1''s are for fail-stop; set -e doesn't work on some systems
 
@@ -54,12 +57,25 @@ rm -f lib/*.o sed/*.o sed/sed
 cd lib || exit 1
 rm -f regex.h
 cp regex_.h regex.h
-${CC} -DHAVE_CONFIG_H -I.. -I. -c \
-  alloca.c getline.c getopt.c getopt1.c memchr.c memcmp.c memmove.c \
-  mkstemp.c strverscmp.c obstack.c regex.c strerror.c utils.c || exit 1
+${CC} -DHAVE_CONFIG_H -I.. -I. -c alloca.c
+${CC} -DHAVE_CONFIG_H -I.. -I. -c getline.c || exit 1
+${CC} -DHAVE_CONFIG_H -I.. -I. -c getopt.c || exit 1
+${CC} -DHAVE_CONFIG_H -I.. -I. -c getopt1.c || exit 1
+${CC} -DHAVE_CONFIG_H -I.. -I. -c memchr.c || exit 1
+${CC} -DHAVE_CONFIG_H -I.. -I. -c memcmp.c || exit 1
+${CC} -DHAVE_CONFIG_H -I.. -I. -c memmove.c || exit 1
+${CC} -DHAVE_CONFIG_H -I.. -I. -c mkstemp.c || exit 1
+${CC} -DHAVE_CONFIG_H -I.. -I. -c strverscmp.c || exit 1
+${CC} -DHAVE_CONFIG_H -I.. -I. -c obstack.c || exit 1
+${CC} -DHAVE_CONFIG_H -I.. -I. -c regex.c || exit 1
+${CC} -DHAVE_CONFIG_H -I.. -I. -c strerror.c || exit 1
+${CC} -DHAVE_CONFIG_H -I.. -I. -c utils.c || exit 1
 
 cd ../sed || exit 1
-${CC} -DHAVE_CONFIG_H -I.. -I. -I../lib -c \
-  sed.c fmt.c compile.c execute.c regex.c || exit 1
+${CC} -DHAVE_CONFIG_H -I.. -I. -I../lib -c sed.c || exit 1
+${CC} -DHAVE_CONFIG_H -I.. -I. -I../lib -c fmt.c || exit 1
+${CC} -DHAVE_CONFIG_H -I.. -I. -I../lib -c compile.c || exit 1
+${CC} -DHAVE_CONFIG_H -I.. -I. -I../lib -c execute.c || exit 1
+${CC} -DHAVE_CONFIG_H -I.. -I. -I../lib -c regexp.c || exit 1
 
 ${CC} -o sed *.o ../lib/*.o || exit 1
