@@ -1,6 +1,6 @@
 
 /*  GNU SED, a batch stream editor.
-    Copyright (C) 1989,90,91,92,93,94,95,98,99,2002,2003
+    Copyright (C) 1989,90,91,92,93,94,95,98,99,2002,2003,2004
     Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,7 @@
 #define INITIAL_BUFFER_SIZE	50
 #define FREAD_BUFFER_SIZE	8192
 
-#include "config.h"
+#include "sed.h"
 
 #include <stdio.h>
 #include <ctype.h>
@@ -54,14 +54,6 @@ extern int errno;
 # include <memory.h>
 #endif
 
-
-#if defined HAVE_WCTYPE_H && defined HAVE_WCHAR_H && defined HAVE_MBRTOWC
-/* We can handle multibyte string.  */
-# include <wchar.h>
-# include <wctype.h>
-# define MBS_SUPPORT
-#endif
-
 #ifndef HAVE_STRCHR
 # define strchr index
 # define strrchr rindex
@@ -80,9 +72,6 @@ extern int errno;
 
 #include <sys/stat.h>
 
-#include "utils.h"
-#include "sed.h"
-
 
 /* Sed operates a line at a time. */
 struct line {
@@ -601,7 +590,14 @@ reset_addresses(vec)
   int n;
 
   for (cur_cmd = vec->v, n = vec->v_length; n--; cur_cmd++)
-    cur_cmd->a1_matched = FALSE;
+    if (cur_cmd->a1)
+      {
+	enum addr_types type = cur_cmd->a1->addr_type;
+        cur_cmd->a1_matched = (type == addr_is_num || type == addr_is_num_mod)
+                              && cur_cmd->a1->addr_number == 0;
+      }
+    else
+      cur_cmd->a1_matched = FALSE;
 }
 
 /* Read in the next line of input, and store it in the pattern space.
@@ -1252,8 +1248,15 @@ execute_program(vec, input)
 
 	    case 'N':
 	      str_append(&line, "\n", 1);
+
 	      if (test_eof(input) || !read_pattern_space(input, vec, TRUE))
-		return -1;
+		{
+		  line.length--;
+                  if (!POSIXLY_CORRECT && !no_default_output)
+                    output_line(line.active, line.length, line.chomped,
+				output_file);
+		  return -1;
+		}
 	      break;
 
 	    case 'p':
@@ -1347,7 +1350,7 @@ execute_program(vec, input)
 
 	    case 'y':
 	      {
-#if defined MBS_SUPPORT && !defined REG_PERL
+#if defined HAVE_MBRTOWC && !defined REG_PERL
 	       if (MB_CUR_MAX > 1)
 		 {
 		   int idx, prev_idx; /* index in the input line.  */
@@ -1416,7 +1419,7 @@ execute_program(vec, input)
 		     }
 		 }
 	       else
-#endif /* MBS_SUPPORT */
+#endif /* HAVE_MBRTOWC */
 		 {
 		   unsigned char *p, *e;
 		   p = CAST(unsigned char *)line.active;
